#include <Basis/Screen.hpp>
#include "LocalLayering.hpp"

#include <Graphics/Graphics.hpp>

#include <SDL.h>
#include <SDL_opengl.h>

#include <Basis/Error.hpp>

int main(int argc, char **argv) {

	Graphics::screen_x = 800;
	Graphics::screen_y = 600;

	unsigned int model_x = 640;
	unsigned int model_y = 480;

	for (int i = 1; i < argc; ++i) {
		string temp = argv[i];
		if (temp.empty() || temp[0] != '-') continue;
		if (temp.size() < 2) continue;
		if (temp[1] == 's' || temp[1] == 'c') {
			unsigned int x = 0;
			unsigned int y = 0;
			bool seen_x = false;
			for (unsigned int i = 2; i < temp.size(); ++i) {
				if (temp[i] >= '0' && temp[i] <= '9') {
					if (seen_x) {
						y *= 10;
						y += temp[i] - '0';
					} else {
						x *= 10;
						x += temp[i] - '0';
					}
				} else if (temp[i] == 'x') {
					seen_x = true;
				} else {
					LOG_WARNING("Weird character '" << temp[i] << "' when parsing resolution.");
				}
			}
			if (x == 0 || y == 0) {
				LOG_WARNING("Ignoring resolution switch '" << temp << "' because of zero in size.");
			} else {
				if (temp[1] == 's') {
					Graphics::screen_x = x;
					Graphics::screen_y = y;
				} else {
					model_x = x;
					model_y = y;
				}
			}
		} else {
			LOG_INFO("-h: help, -sNNNxNNN screen size, -cNNNxNNN canvas size (if you are doing 3d)");
			exit(0);
		}
	}

	if (SDL_Init(SDL_INIT_TIMER) != 0) {
		LOG_ERROR("Could not initialize sdl: " << SDL_GetError());
		return 1;
	}

	if (!Graphics::init(Graphics::NEED_STENCIL)) {
		LOG_ERROR("Could not initialize graphics, not continuing.");
		exit(1);
	}

	//Some quick basic GL state:
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glScalef(1.0f / Graphics::aspectf(), 1.0f, 1.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);

	//we'll draw our own cursor, thank you.
	SDL_ShowCursor(SDL_DISABLE);
	//if (argc > 2) {
	//	SDL_WM_GrabInput(SDL_GRAB_ON);
	//}
	
	SDL_EnableUNICODE(1);

	LocalLayering m;
	for (int i = 1; i < argc; ++i) {
		string temp = argv[i];
		if (!temp.empty() && temp[0] == '-') continue; //we parsed these already.. I hope.
		if (temp.size() > 4 && temp.substr(temp.size()-4,4) == ".obj") {
			m.add_model(temp);
		} else {
			m.add_layer(temp);
		}
	}
	if (m.width == 0 && m.height == 0) {
		m.add_layer("d1.png");
		m.add_layer("d2.png");
	}
	if (m.width == 0 && m.height == 0) {
		if (m.model.verts.empty()) {
			SDL_Quit();
			return 0;
		}
		m.width = model_x;
		m.height = model_y;
	}

	start_screen(&m);

	Graphics::deinit();

	SDL_Quit();

	return 0;
}
